#include <MsTimer2.h>

#include <Arduino.h>

// FMS PIC protocol for connecting a PPM stream to a PC.
// http://modelsimulator.com/
//
// Implements the 19200 baud FMS PIC protocol
// Converted from code by
// Richard.Prinz@MIN.at 3.2013
//
// This sketch will find the syncpulse and encode it as 0xFF, and encode
// the individual channels in sequence with the proportional value for each
// channel from 0x00 to 0xFE.
// The code will continue to encode channels regardless of their number
// until the next syncpulse.
//
// The PPM sigmal from the transmitter (or receiver, if it can suply a PPM signal)
// is connected to pin 3, external interupt 1. The Arduino board communicates with
// the computer through the serial port, thus being able to connect throug USB
// interface.
//
// The code uses the timer interrupt through the MsTimer2 library. This can be omitted
// if not wanted. It just adds the capability to send a "heartbeat" signal to the
// simulator if no PPM signal is detected for a while.
// http://playground.arduino.cc/Main/MsTimer2
//
// by Reynir Siik
// 2015-MAR-14, Pi day
//

#define MIN_MS            1000ul
/*
#define MID_MS            1500
#define MAX_MS            2000
#define RANGE_MS          1000
*/
#define CO                3.9215686
// 1000/255 == 3.9215686

// Variables used in interupt routine is declared volatile to survive task switching
volatile boolean nextPtr = 0;
volatile unsigned long time[2];
volatile boolean sem = false;
volatile byte numWDT = 0;

void setup()
{
  Serial.begin(19200);
  MsTimer2::set(100, ISR_WDTimer);   // 100 ms timeout
  MsTimer2::start();
  attachInterrupt(1, ISR1, FALLING); //Trigg on negative edge on pin 3
  /*
  PPM pulse train:
   __   ___________________________________   _____   ____   _______   ___ //___   __________   __ HIGH
     |_|      Sync pulse > 3000 us         |_|     |_|    |_|       |_|   //    |_|          |_|
                                               Channel pulse 1000 to 2000 us                       LOW
   Pulse start and end on falling edge. The end of one pulse is the begining of next.
   Sync pulse translates to FF
   Channel pulse length is encoded in the range 00 to FE, 1000 us is 00 and 2000 us is FE
   */
}

void loop()
{
  if (sem == true)
  {
    sem = false;
    int r = (time[!nextPtr] - time[nextPtr] - MIN_MS);
    if (r > 3000)       //sync pulse code 255
      r = 255;
    else {              //channels encoded in range 0-254
      r = (int)(r / CO);
      if (r > 254)
        r = 254;
    }

    Serial.write(r);    // output FMS protocol
    numWDT = 0;
  }

  if (numWDT > 2)       // watch dog timeout
  {
    numWDT = 0;
    Serial.write(255);  // output FMS protocol
  }
}

/* Pulse timing interupt routine */
void ISR1()
{
  time[nextPtr] = micros();
  nextPtr = !nextPtr;
  sem = true;
}

/* Watch dog timer interupt routine */
void ISR_WDTimer() {
  numWDT++;
}
